@@ -1,4 +1,4 @@
-/* 
+/*
  * Copyright (C) 2012-2012 Cheer Xiao
  * Copyright (C) 2012-2012 CSSlayer
  *
@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc.
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "fcitx-m17n.h"
@@ -23,4 +23,5 @@
 CONFIG_BINDING_BEGIN(FcitxM17NConfig)
 CONFIG_BINDING_REGISTER("M17N", "PrevPage", hkPrevPage)
 CONFIG_BINDING_REGISTER("M17N", "NextPage", hkNextPage)
+CONFIG_BINDING_REGISTER("M17N", "EnableDeprecated", enableDeprecated)
 CONFIG_BINDING_END()
