@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc.
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -41,6 +41,7 @@
 
 #include "fcitx-m17n.h"
 #include "keysymname.h"
+#include "overrideparser.h"
 
 #define CONF_FNAME "fcitx-m17n.config"
 #define TEXTDOMAIN "fcitx-m17n"
@@ -49,7 +50,6 @@
 static const FcitxHotkey FCITX_M17N_UP[2] = {{NULL, FcitxKey_Up, 0}, {NULL, FcitxKey_P, FcitxKeyState_Ctrl}};
 static const FcitxHotkey FCITX_M17N_DOWN[2] = {{NULL, FcitxKey_Down, 0}, {NULL, FcitxKey_N, FcitxKeyState_Ctrl}};
 
-extern const char*        KeySymName (FcitxKeySym keyval);
 static MSymbol            KeySymToSymbol (FcitxKeySym sym, unsigned int state);
 
 static void*              FcitxM17NCreate(FcitxInstance *instance);
@@ -76,16 +76,11 @@ static void*              MPListIndex(MPlist *head, size_t idx);
 inline static void        SetPreedit(FcitxInstance* inst, FcitxInputState* is, const char* s, int cursor_pos);
 static int                GetPageSize(MSymbol mlang, MSymbol mname);
 
-
-FCITX_EXPORT_API
-FcitxIMClass ime = {
+FCITX_DEFINE_PLUGIN(fcitx_m17n, ime, FcitxIMClass) = {
     FcitxM17NCreate,
     FcitxM17NDestroy
 };
 
-FCITX_EXPORT_API
-int ABI_VERSION = FCITX_ABI_VERSION;
-
 char* MTextToUTF8(MText* mt)
 {
     // TODO Verify that bufsize is "just enough" in worst scenerio.
@@ -224,7 +219,7 @@ FcitxIRV FcitxM17NGetCandWords(void *arg)
 
     if (im->owner->mic->status) {
         char* mstatus = MTextToUTF8(im->owner->mic->status);
-        toShow = toShow || (strlen(mstatus) != 0);
+        // toShow = toShow || (strlen(mstatus) != 0);
         if (strlen(mstatus) != 0) {
             FcitxLog(DEBUG, "IM status changed to %s", mstatus);
             // FcitxMessages* auxDown = FcitxInputStateGetAuxUpDown(is);
@@ -267,7 +262,7 @@ FcitxIRV FcitxM17NGetCandWords(void *arg)
                 char *words = MTextToUTF8((MText*) mplist_value(head));
                 char *p, *q;
                 for (p = words; *p; p = q) {
-                    int unused;
+                    uint32_t unused;
                     q = fcitx_utf8_get_char(p, &unused);
                     cand.strWord = strndup(p, q-p);
                     cand.priv = fcitx_utils_malloc0(sizeof(int));
@@ -284,14 +279,10 @@ FcitxIRV FcitxM17NGetCandWords(void *arg)
         }
         toShow = toShow || hasCand;
     }
-    if (toShow) {
-        ret |= IRV_DISPLAY_CANDWORDS;
-    }
-
-
+    FcitxUIUpdateInputWindow(inst);
 
     if (im->forward) {
-        ret |= IRV_FLAG_FORWARD_KEY;
+        ret = IRV_TO_PROCESS;
     }
     return ret;
 }
@@ -324,6 +315,7 @@ MSymbol KeySymToSymbol (FcitxKeySym sym, unsigned int state)
     }
 
     const char* base = "";
+    char temp[2] = " ";
 
     if (sym >= FcitxKey_space && sym <= FcitxKey_asciitilde) {
         FcitxKeySym c = sym;
@@ -337,7 +329,6 @@ MSymbol KeySymToSymbol (FcitxKeySym sym, unsigned int state)
             mask |= FcitxKeyState_Ctrl;
         }
 
-        char temp[2] = " ";
         temp[0] = c & 0xff;
         base = temp;
     }
@@ -356,7 +347,7 @@ MSymbol KeySymToSymbol (FcitxKeySym sym, unsigned int state)
     if (mask & FcitxKeyState_Hyper) {
         prefix = "H-";
     }
-    if (mask & FcitxKeyState_Super2) {
+    if (mask & FcitxKeyState_Super) {
         prefix = "s-";
     }
     // This is mysterious. - xiaq
@@ -418,10 +409,12 @@ FcitxIRV FcitxM17NDoInput(void* arg, FcitxKeySym sym, unsigned state)
 
 FcitxIRV FcitxM17NDoInputInternal(IM* im, FcitxKeySym sym, unsigned state)
 {
-    MSymbol msym = KeySymToSymbol(sym, state);
     FcitxInstance* inst = im->owner->owner;
+    FcitxInputState* is = FcitxInstanceGetInputState(inst);
     FcitxInputContext* ic = FcitxInstanceGetCurrentIC(inst);
 
+    MSymbol msym = KeySymToSymbol(sym, state);
+
     if (msym == Mnil) {
         FcitxLog(DEBUG, "sym=%x, state=%x, not my dish", sym, state);
         return IRV_TO_PROCESS;
@@ -530,6 +523,14 @@ void *FcitxM17NCreate(FcitxInstance* inst)
     // but it also makes the the code a little simpler.
     addon->ims = (IM**) fcitx_utils_malloc0(sizeof(IM*) * addon->nim);
 
+    OverrideItemList* list = NULL;
+    FILE* fp = FcitxXDGGetFileWithPrefix("m17n", "default", "r", NULL);
+    if (fp) {
+        list = ParseDefaultSettings(fp);
+        fclose(fp);
+    }
+
+    char* curlang = fcitx_utils_get_current_langcode();
     int i;
     for (i = 0; i < addon->nim; i++, mimlist = mplist_next(mimlist)) {
         // See m17n documentation of minput_list() in input.c.
@@ -542,9 +543,16 @@ void *FcitxM17NCreate(FcitxInstance* inst)
         char *lang = msymbol_name(mlang);
         char *name = msymbol_name(mname);
 
+        OverrideItem* item = NULL;
+        if (list)
+            item = MatchDefaultSettings(list, lang, name);
+
+        if (item && item->priority < 0 && !addon->config.enableDeprecated)
+            continue;
+
         if (msane != Mt) {
             // Not "sane"
-            FcitxLog(WARNING, "Insane IM [%s: %s]", lang, name);
+            // FcitxLog(WARNING, "Insane IM [%s: %s]", lang, name);
             continue;
         }
 
@@ -557,14 +565,13 @@ void *FcitxM17NCreate(FcitxInstance* inst)
         }
 
         if (!(addon->ims[i] = FcitxM17NMakeIM(addon, mlang, mname))) {
-            FcitxLog(ERROR, "Failed to create IM [%s: %s]", lang, name);
             continue;
         }
-        FcitxLog(INFO, "Created IM [%s: %s]", lang, name);
+        FcitxLog(DEBUG, "Created IM [%s: %s]", lang, name);
 
         char *uniqueName, *fxName, *iconName;
         asprintf(&uniqueName, "m17n_%s_%s", lang, name);
-        asprintf(&fxName, _("%s - %s (M17N)"), lang, name);
+        asprintf(&fxName, _("%s (M17N)"), (item && item->i18nName) ? _(item->i18nName) : name);
 
         info = minput_get_title_icon(mlang, mname);
         // head of info is a MText
@@ -577,18 +584,29 @@ void *FcitxM17NCreate(FcitxInstance* inst)
             // /usr/share/m17n/icons/... on Linux systems, this is a
             // reasonable assumption.
             iconName = MTextToUTF8(iconPath);
-            FcitxLog(INFO, "Mim icon is %s", iconName);
+            FcitxLog(DEBUG, "Mim icon is %s", iconName);
         } else {
             iconName = uniqueName;
         }
 
-        FcitxInstanceRegisterIM(
+        FcitxIMIFace iface;
+        memset(&iface, 0, sizeof(FcitxIMIFace));
+        iface.Init = FcitxM17NInit;
+        iface.DoInput = FcitxM17NDoInput;
+        iface.ResetIM = FcitxM17NReset;
+        iface.Save = FcitxM17NSave;
+        iface.ReloadConfig = FcitxM17NReload;
+        iface.GetCandWords = FcitxM17NGetCandWords;
+
+        int priority = 100;
+        if (item && strncmp(curlang, lang, 2) == 0 && item->priority > 0)
+            priority = item->priority;
+
+        FcitxInstanceRegisterIMv2(
             inst, addon->ims[i],
             uniqueName, fxName, iconName,
-            FcitxM17NInit, FcitxM17NReset, FcitxM17NDoInput,
-            FcitxM17NGetCandWords, NULL, FcitxM17NSave,
-            FcitxM17NReload, NULL,
-            100, strcmp(lang, "t") == 0 ? "*" : lang
+            iface,
+            priority, strcmp(lang, "t") == 0 ? "*" : lang
         );
         if (iconName != uniqueName) {
             free(iconName);
@@ -597,6 +615,11 @@ void *FcitxM17NCreate(FcitxInstance* inst)
         free(fxName);
     }
 
+    fcitx_utils_free(curlang);
+
+    if (list)
+        utarray_free(list);
+
     return addon;
 }
 
@@ -714,4 +737,4 @@ FcitxM17NCallback (MInputContext *context,
         else if (len > 0)
             FcitxInstanceDeleteSurroundingText (im->owner->owner, ic, 0, -len);
     }
-}
\ No newline at end of file
+}
